@@ -1,25 +1,30 @@
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Implementing communication with MySQL servers.
 """
@@ -29,7 +34,7 @@ import os
 import time
 
 from .authentication import get_auth_plugin
-from .catch23 import PY2, isstr
+from .catch23 import PY2, isstr, UNICODE_TYPES
 from .constants import (
     ClientFlag, ServerCmd, ServerFlag,
     flag_is_set, ShutdownType, NET_BUFFER_LENGTH
@@ -78,7 +83,6 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._use_unicode = True
         self._get_warnings = False
         self._raise_on_warnings = False
-        self._connection_timeout = None
         self._buffered = False
         self._unread_result = False
         self._have_next_result = False
@@ -91,8 +95,14 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._auth_plugin = None
         self._pool_config_version = None
 
-        if len(kwargs) > 0:
-            self.connect(**kwargs)
+        if kwargs:
+            try:
+                self.connect(**kwargs)
+            except:
+                # Tidy-up underlying socket on failure
+                self.close()
+                self._socket = None
+                raise
 
     def _do_handshake(self):
         """Get the handshake from the MySQL server"""
@@ -110,6 +120,14 @@ class MySQLConnection(MySQLConnectionAbstract):
         self._server_version = self._check_server_version(
             handshake['server_version_original'])
 
+        if not handshake['capabilities'] & ClientFlag.SSL:
+            self._client_flags &= ~ClientFlag.SSL
+            if self._ssl.get('verify_cert'):
+                raise errors.InterfaceError("SSL is required but the server "
+                                            "doesn't support it", errno=2026)
+        elif not self._ssl_disabled:
+            self._client_flags |= ClientFlag.SSL
+
         if handshake['capabilities'] & ClientFlag.PLUGIN_AUTH:
             self.set_client_flags([ClientFlag.PLUGIN_AUTH])
 
@@ -127,11 +145,15 @@ class MySQLConnection(MySQLConnectionAbstract):
         reply back. Raises any error coming from MySQL.
         """
         self._ssl_active = False
-        if client_flags & ClientFlag.SSL and ssl_options:
+        if client_flags & ClientFlag.SSL:
             packet = self._protocol.make_auth_ssl(charset=charset,
                                                   client_flags=client_flags)
             self._socket.send(packet)
-            self._socket.switch_to_ssl(**ssl_options)
+            self._socket.switch_to_ssl(ssl_options.get('ca'),
+                                       ssl_options.get('cert'),
+                                       ssl_options.get('key'),
+                                       ssl_options.get('verify_cert') or False,
+                                       ssl_options.get('cipher'))
             self._ssl_active = True
 
         packet = self._protocol.make_auth(
@@ -154,6 +176,8 @@ class MySQLConnection(MySQLConnectionAbstract):
         Raises NotSupportedError when we get the old, insecure password
         reply back. Raises any error coming from MySQL.
         """
+        auth = None
+        new_auth_plugin = self._auth_plugin or self._handshake["auth_plugin"]
         packet = self._socket.recv()
         if packet[4] == 254 and len(packet) == 5:
             raise errors.NotSupportedError(
@@ -169,12 +193,22 @@ class MySQLConnection(MySQLConnectionAbstract):
             response = auth.auth_response()
             self._socket.send(response)
             packet = self._socket.recv()
-            if packet[4] != 1:
-                return self._handle_ok(packet)
-            else:
-                auth_data = self._protocol.parse_auth_more_data(packet)
+
+        if packet[4] == 1:
+            auth_data = self._protocol.parse_auth_more_data(packet)
+            auth = get_auth_plugin(new_auth_plugin)(
+                auth_data, password=password, ssl_enabled=self._ssl_active)
+            if new_auth_plugin == "caching_sha2_password":
+                response = auth.auth_response()
+                if response:
+                    self._socket.send(response)
+                packet = self._socket.recv()
+
+        if packet[4] == 0:
+            return self._handle_ok(packet)
         elif packet[4] == 255:
             raise errors.get_exception(packet)
+        return None
 
     def _get_connection(self, prtcls=None):
         """Get connection based on configuration
@@ -191,6 +225,7 @@ class MySQLConnection(MySQLConnectionAbstract):
             conn = MySQLTCPSocket(host=self.server_host,
                                   port=self.server_port,
                                   force_ipv6=self._force_ipv6)
+
         conn.set_connection_timeout(self._connection_timeout)
         return conn
 
@@ -203,15 +238,20 @@ class MySQLConnection(MySQLConnectionAbstract):
         """
         self._protocol = MySQLProtocol()
         self._socket = self._get_connection()
-        self._socket.open_connection()
-        self._do_handshake()
-        self._do_auth(self._user, self._password,
-                      self._database, self._client_flags, self._charset_id,
-                      self._ssl)
-        self.set_converter_class(self._converter_class)
-        if self._client_flags & ClientFlag.COMPRESS:
-            self._socket.recv = self._socket.recv_compressed
-            self._socket.send = self._socket.send_compressed
+        try:
+            self._socket.open_connection()
+            self._do_handshake()
+            self._do_auth(self._user, self._password,
+                          self._database, self._client_flags, self._charset_id,
+                          self._ssl)
+            self.set_converter_class(self._converter_class)
+            if self._client_flags & ClientFlag.COMPRESS:
+                self._socket.recv = self._socket.recv_compressed
+                self._socket.send = self._socket.send_compressed
+        except:
+            # close socket
+            self.close()
+            raise
 
     def shutdown(self):
         """Shut down connection to MySQL Server.
@@ -231,13 +271,14 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         try:
             self.cmd_quit()
-            self._socket.close_connection()
         except (AttributeError, errors.Error):
             pass  # Getting an exception would mean we are disconnected.
+        self._socket.close_connection()
+
     disconnect = close
 
     def _send_cmd(self, command, argument=None, packet_number=0, packet=None,
-                  expect_response=True):
+                  expect_response=True, compressed_packet_number=0):
         """Send a command to the MySQL server
 
         This method sends a command with an optional argument.
@@ -258,7 +299,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         try:
             self._socket.send(
                 self._protocol.make_command(command, packet or argument),
-                packet_number)
+                packet_number, compressed_packet_number)
         except AttributeError:
             raise errors.OperationalError("MySQL Connection not available.")
 
@@ -325,7 +366,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         """
         if packet[4] == 0:
             ok_pkt = self._protocol.parse_ok(packet)
-            self._handle_server_status(ok_pkt['server_status'])
+            self._handle_server_status(ok_pkt['status_flag'])
             return ok_pkt
         elif packet[4] == 255:
             raise errors.get_exception(packet)
@@ -419,7 +460,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         Returns a tuple.
         """
         (rows, eof) = self.get_rows(count=1, binary=binary, columns=columns)
-        if len(rows):
+        if rows:
             return (rows[0], eof)
         return (None, eof)
 
@@ -437,15 +478,23 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         try:
             if binary:
+                charset = self.charset
+                if charset == 'utf8mb4':
+                    charset = 'utf8'
                 rows = self._protocol.read_binary_result(
-                    self._socket, columns, count)
+                    self._socket, columns, count, charset)
             else:
-                rows = self._protocol.read_text_result(self._socket, count)
+                rows = self._protocol.read_text_result(self._socket,
+                                                       self._server_version,
+                                                       count=count)
         except errors.Error as err:
             self.unread_result = False
             raise err
+
         if rows[-1] is not None:
-            self._handle_server_status(rows[-1]['status_flag'])
+            row = rows[-1]  # OK or EOF
+            self._handle_server_status(row['status_flag'] if 'status_flag' in
+                                       row else row['server_status'])
             self.unread_result = False
 
         return rows
@@ -511,10 +560,9 @@ class MySQLConnection(MySQLConnectionAbstract):
         Returns a generator.
         """
         if not isinstance(statements, bytearray):
-            if isstr(statements):
-                statements = bytearray(statements.encode('utf-8'))
-            else:
-                statements = bytearray(statements)
+            if isstr(statements) and isinstance(statements, UNICODE_TYPES):
+                statements = statements.encode('utf8')
+            statements = bytearray(statements)
 
         # Handle the first query result
         yield self._handle_result(self._send_cmd(ServerCmd.QUERY, statements))
@@ -553,7 +601,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self.handle_unread_result()
 
         packet = self._protocol.make_command(ServerCmd.QUIT)
-        self._socket.send(packet, 0)
+        self._socket.send(packet, 0, 0)
         return packet
 
     def cmd_shutdown(self, shutdown_type=None):
@@ -587,7 +635,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self.handle_unread_result()
 
         packet = self._protocol.make_command(ServerCmd.STATISTICS)
-        self._socket.send(packet, 0)
+        self._socket.send(packet, 0, 0)
         return self._protocol.parse_statistics(self._socket.recv())
 
     def cmd_process_kill(self, mysql_pid):
@@ -646,7 +694,7 @@ class MySQLConnection(MySQLConnectionAbstract):
             charset=charset, client_flags=self._client_flags,
             ssl_enabled=self._ssl_active,
             auth_plugin=self._auth_plugin)
-        self._socket.send(packet, 0)
+        self._socket.send(packet, 0, 0)
 
         ok_packet = self._auth_switch_request(username, password)
 
@@ -927,6 +975,17 @@ class MySQLConnection(MySQLConnectionAbstract):
         eof = self._handle_eof(self._socket.recv())
         return (column_count, columns, eof)
 
+    def cmd_stmt_fetch(self, statement_id, rows=1):
+        """Fetch a MySQL statement Result Set
+
+        This method will send the FETCH command to MySQL together with the
+        given statement id and the number of rows to fetch.
+        """
+        packet = self._protocol.make_stmt_fetch(statement_id, rows)
+        self.unread_result = False
+        self._send_cmd(ServerCmd.STMT_FETCH, packet, expect_response=False)
+        self.unread_result = True
+
     def cmd_stmt_prepare(self, statement):
         """Prepare a MySQL statement
 
