@@ -1,25 +1,30 @@
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Cursor classes
 """
@@ -188,7 +193,7 @@ class CursorBase(MySQLCursorAbstract):
         """
         pass
 
-    def executemany(self, operation, seqparams):
+    def executemany(self, operation, seq_params):
         """Execute the given operation multiple times
 
         The executemany() method will execute the operation iterating
@@ -493,7 +498,7 @@ class MySQLCursor(CursorBase):
 
         i = 0
         while True:
-            result = next(query_iter)
+            result = next(query_iter)  # pylint: disable=R1708
             self._reset_result()
             self._handle_result(result)
             try:
@@ -556,15 +561,15 @@ class MySQLCursor(CursorBase):
         if multi:
             self._executed_list = []
             return self._execute_iter(self._connection.cmd_query_iter(stmt))
-        else:
-            try:
-                self._handle_result(self._connection.cmd_query(stmt))
-            except errors.InterfaceError:
-                if self._connection._have_next_result:  # pylint: disable=W0212
-                    raise errors.InterfaceError(
-                        "Use multi=True when executing multiple statements")
-                raise
-            return None
+
+        try:
+            self._handle_result(self._connection.cmd_query(stmt))
+        except errors.InterfaceError:
+            if self._connection._have_next_result:  # pylint: disable=W0212
+                raise errors.InterfaceError(
+                    "Use multi=True when executing multiple statements")
+            raise
+        return None
 
     def _batch_insert(self, operation, seq_params):
         """Implements multi row insert"""
@@ -577,8 +582,7 @@ class MySQLCursor(CursorBase):
             """
             if match.group(1):
                 return ""
-            else:
-                return match.group(2)
+            return match.group(2)
 
         tmp = re.sub(RE_SQL_ON_DUPLICATE, '',
                      re.sub(RE_SQL_COMMENT, remove_comments, operation))
@@ -612,8 +616,7 @@ class MySQLCursor(CursorBase):
                 stmt = stmt.replace(fmt, b','.join(values), 1)
                 self._executed = stmt
                 return stmt
-            else:
-                return None
+            return None
         except (UnicodeDecodeError, UnicodeEncodeError) as err:
             raise errors.ProgrammingError(str(err))
         except errors.Error:
@@ -658,7 +661,7 @@ class MySQLCursor(CursorBase):
         if re.match(RE_SQL_INSERT_STMT, operation):
             if not seq_params:
                 self._rowcount = 0
-                return
+                return None
             stmt = self._batch_insert(operation, seq_params)
             if stmt is not None:
                 return self.execute(stmt)
@@ -677,6 +680,7 @@ class MySQLCursor(CursorBase):
             # Raise whatever execute() raises
             raise
         self._rowcount = rowcnt
+        return None
 
     def stored_results(self):
         """Returns an iterator for stored results
@@ -758,12 +762,10 @@ class MySQLCursor(CursorBase):
             can_consume_results = self._connection._consume_results
             for result in self._connection.cmd_query_iter(call):
                 self._connection._consume_results = False
-                # pylint: disable=R0204
                 if self._raw:
                     tmp = MySQLCursorBufferedRaw(self._connection._get_self())
                 else:
                     tmp = MySQLCursorBuffered(self._connection._get_self())
-                # pylint: enable=R0204
                 tmp._executed = "(a result of {0})".format(call)
                 tmp._handle_result(result)
                 if tmp._warnings is not None:
@@ -778,9 +780,9 @@ class MySQLCursor(CursorBase):
                 self.execute(select)
                 self._stored_results = results
                 return self.fetchone()
-            else:
-                self._stored_results = results
-                return ()
+
+            self._stored_results = results
+            return ()
 
         except errors.Error:
             raise
@@ -815,7 +817,7 @@ class MySQLCursor(CursorBase):
             raise errors.InterfaceError(
                 "Failed getting warnings; %s" % err)
 
-        if len(res):
+        if res:
             return res
 
         return None
@@ -1113,7 +1115,7 @@ class MySQLCursorPrepared(MySQLCursor):
                                            eof.get('server_status', 0)))
         super(MySQLCursorPrepared, self)._handle_eof(eof)
 
-    def callproc(self, *args, **kwargs):
+    def callproc(self, procname, args=()):
         """Calls a stored procedue
 
         Not supported with MySQLCursorPrepared.
@@ -1137,21 +1139,21 @@ class MySQLCursorPrepared(MySQLCursor):
         """
         pass
 
-    def _handle_result(self, res):
+    def _handle_result(self, result):
         """Handle result after execution"""
-        if isinstance(res, dict):
+        if isinstance(result, dict):
             self._connection.unread_result = False
             self._have_result = False
-            self._handle_noresultset(res)
+            self._handle_noresultset(result)
         else:
-            self._description = res[1]
+            self._description = result[1]
             self._connection.unread_result = True
             self._have_result = True
 
-            if 'status_flag' in res[2]:
-                self._handle_server_status(res[2]['status_flag'])
-            elif 'server_status' in res[2]:
-                self._handle_server_status(res[2]['server_status'])
+            if 'status_flag' in result[2]:
+                self._handle_server_status(result[2]['status_flag'])
+            elif 'server_status' in result[2]:
+                self._handle_server_status(result[2]['server_status'])
 
     def execute(self, operation, params=(), multi=False):  # multi is unused
         """Prepare and execute a MySQL Prepared Statement
@@ -1169,7 +1171,10 @@ class MySQLCursorPrepared(MySQLCursor):
             self._executed = operation
             try:
                 if not isinstance(operation, bytes):
-                    operation = operation.encode(self._connection.charset)
+                    charset = self._connection.charset
+                    if charset == 'utf8mb4':
+                        charset = 'utf8'
+                    operation = operation.encode(charset)
             except (UnicodeDecodeError, UnicodeEncodeError) as err:
                 raise errors.ProgrammingError(str(err))
 
@@ -1342,6 +1347,7 @@ class MySQLCursorNamedTuple(MySQLCursor):
                 NAMED_TUPLE_CACHE[columns] = named_tuple
             # pylint: enable=W0201
             return named_tuple(*row)
+        return None
 
     def fetchone(self):
         """Returns next row of a query result set
@@ -1350,8 +1356,7 @@ class MySQLCursorNamedTuple(MySQLCursor):
         if row:
             if hasattr(self._connection, 'converter'):
                 return self._row_to_python(row, self.description)
-            else:
-                return row
+            return row
         return None
 
     def fetchall(self):
