@@ -1,25 +1,30 @@
-# MySQL Connector/Python - MySQL driver written in Python.
-# Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
-
-# MySQL Connector/Python is licensed under the terms of the GPLv2
-# <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
-# MySQL Connectors. There are special exceptions to the terms and
-# conditions of the GPLv2 as it is applied to this software, see the
-# FOSS License Exception
-# <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
+# Copyright (c) 2013, 2018, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation.
+# it under the terms of the GNU General Public License, version 2.0, as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms,
+# as designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an
+# additional permission to link the program and your derivative works
+# with the separately licensed software that they have included with
+# MySQL.
+#
+# Without limiting anything contained in the foregoing, this file,
+# which is part of MySQL Connector/Python, is also subject to the
+# Universal FOSS Exception, version 1.0, a copy of which can be found at
+# http://oss.oracle.com/licenses/universal-foss-exception.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+# See the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
 
 """Implementing caching mechanisms for MySQL Fabric"""
 
@@ -116,32 +121,32 @@ class CacheShardTable(CacheEntry):
     def add_partition(self, key, group):
         """Add sharding information for a group"""
         if self.shard_type == 'RANGE':
-            key = int(key)
+            _key = int(key)
         elif self.shard_type == 'RANGE_DATETIME':
             try:
                 if ':' in key:
-                    key = datetime.strptime(key, "%Y-%m-%d %H:%M:%S")
+                    _key = datetime.strptime(key, "%Y-%m-%d %H:%M:%S")
                 else:
-                    key = datetime.strptime(key, "%Y-%m-%d").date()
+                    _key = datetime.strptime(key, "%Y-%m-%d").date()
             except:
                 raise ValueError(
                     "RANGE_DATETIME key could not be parsed, was: {0}".format(
                         key
                     ))
         elif self.shard_type == 'RANGE_STRING':
-            pass
+            _key = key
         elif self.shard_type == "HASH":
-            pass
+            _key = key
         else:
             raise ValueError("Unsupported sharding type {0}".format(
                 self.shard_type
             ))
-        self.partitioning[key] = {
+        self.partitioning[_key] = {
             'group': group,
         }
         self.reset_ttl()
-        bisect.insort_right(self.keys, key)
-        insort_right_rev(self.keys_reversed, key)
+        bisect.insort_right(self.keys, _key)
+        insort_right_rev(self.keys_reversed, _key)
 
     @classmethod
     def hash_index(cls, part1, part2=None):
@@ -175,6 +180,7 @@ class CacheGroup(CacheEntry):
             group=self.group_name,
         )
 
+
 class FabricCache(object):
     """Singleton class for caching Fabric data
 
